"""cmudict

A versioned python wrapper package for The CMU Pronouncing Dictionary data
files. Compatible with NLTK's CMUDictCorpusReader.
"""

import re
import sys
from collections import defaultdict

if sys.version_info >= (3, 9):
    from importlib import metadata, resources
else:
    import importlib_metadata as metadata
    import importlib_resources as resources

__version__ = metadata.version(__name__)

CMUDICT_PATH = "data"
CMUDICT_DICT = "cmudict.dict"
CMUDICT_PHONES = "cmudict.phones"
CMUDICT_SYMBOLS = "cmudict.symbols"
CMUDICT_VP = "cmudict.vp"
CMUDICT_LICENSE = "LICENSE"


def _stream(resource_name):
    stream = resources.files(__name__).joinpath(CMUDICT_PATH).joinpath(resource_name)
    with stream.open("rb") as file:
        return file.readlines()


def _string(resource_name):
    stream = _stream(resource_name)
    string = "".join([line.decode("utf-8") for line in stream])
    return string


def _entries(stream, comment_string=None):
    cmudict_entries = []
    for line in stream:
        parts = []
        if comment_string:
            parts = line.decode("utf-8").strip().split(comment_string)[0].split()
        else:
            parts = line.decode("utf-8").strip().split()
        thing = re.sub(r"\(\d+\)$", "", parts[0])
        cmudict_entries.append((thing, parts[1:]))
    return cmudict_entries


# pylint: disable-next=redefined-builtin
def dict():
    """
    Compatibility with NLTK.
    Returns the cmudict lexicon as a dictionary, whose keys are
    lowercase words and whose values are lists of pronunciations.
    """
    default = defaultdict(list)
    for key, value in entries():
        default[key].append(value)
    return default


def dict_stream():
    """Return a readable file-like object of the cmudict.dict file."""
    stream = _stream(CMUDICT_DICT)
    return stream


def dict_string():
    """Return the contents of cmudict.dict as a string."""
    string = _string(CMUDICT_DICT)
    return string


def license_string():
    """Return the contents of LICENSE as a string."""
    string = _string(CMUDICT_LICENSE)
    return string


def phones():
    """Return a list of phones used in the main dict."""
    cmu_phones = []
    for line in phones_stream():
        parts = line.decode("utf-8").strip().split()
        cmu_phones.append((parts[0], parts[1:]))
    return cmu_phones


def phones_stream():
    """Return a readable file-like object of the cmudict.phones file."""
    p_stream = _stream(CMUDICT_PHONES)
    return p_stream


def phones_string():
    """Return the contents of cmudict.phones as a string."""
    string = _string(CMUDICT_PHONES)
    return string


def symbols():
    """Return a list of symbols."""
    cmu_symbols = []
    for line in symbols_stream():
        cmu_symbols.append(line.decode("utf-8").strip())
    return cmu_symbols


def symbols_stream():
    """Return a readable file-like object of the cmudict.symbols file."""
    stream = _stream(CMUDICT_SYMBOLS)
    return stream


def symbols_string():
    """Return the contents of cmudict.symbols as a string."""
    string = _string(CMUDICT_SYMBOLS)
    return string


# pylint: disable-next=invalid-name
def vp():
    """Return a list of punctuation pronounciations."""
    cmu_vp = defaultdict(list)
    for key, value in _entries(vp_stream()):
        cmu_vp[key].append(value)
    return cmu_vp


def vp_stream():
    """Return a readable file-like object of the cmudict.vp file."""
    stream = _stream(CMUDICT_VP)
    return stream


def vp_string():
    """Return the contents of cmudict.vp as a string."""
    string = _string(CMUDICT_VP)
    return string


# The .entries(), .raw(), and .words() functions
# maintain compatability with NTLK.
def entries():
    """
    Compatibility with NLTK.
    Returns the cmudict lexicon as a list of entries
    containing (word, transcriptions) tuples.
    """
    cmu_entries = _entries(dict_stream(), "#")
    return cmu_entries


def raw():
    """
    Compatibility with NLTK.
    Returns the cmudict lexicon as a raw string.
    """
    string = _string(CMUDICT_DICT)
    return string


def words():
    """
    Compatibility with NLTK.
    Returns a list of all words defined in the cmudict lexicon.
    """
    return [word.lower() for (word, _) in entries()]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cmudict']

package_data = \
{'': ['*'], 'cmudict': ['data/*']}

install_requires = \
['importlib-metadata>=5.1.0,<6.0.0', 'importlib-resources>=5.10.1,<6.0.0']

setup_kwargs = {
    'name': 'cmudict',
    'version': '1.0.7',
    'description': 'A versioned python wrapper package for The CMU Pronouncing Dictionary data files.',
    'long_description': "# CMUdict: Python wrapper for cmudict\n\n[![Latest PyPI version](https://img.shields.io/pypi/v/cmudict.svg)](https://pypi.python.org/pypi/cmudict)\n[![GitHub Workflow Status](https://github.com/prosegrinder/python-cmudict/workflows/Python%20CI/badge.svg?branch=main)](https://github.com/prosegrinder/python-cmudict/actions?query=workflow%3A%22Python+CI%22+branch%3Amain)\n\nCMUdict is a versioned python wrapper package for\n[The CMU Pronouncing Dictionary](https://github.com/cmusphinx/cmudict) data\nfiles. The main purpose is to expose the data with little or no assumption on\nhow it is to be used.\n\n## Installation\n\n`cmudict` is available on PyPI. Simply install it with `pip`:\n\n```bash\npip install cmudict\n```\n\n## Usage\n\nThe cmudict data set includes 4 data files: cmudict.dict, cmudict.phones,\ncmudict.symbols, and cmudict.vp. See\n[The CMU Pronouncing Dictionary](https://github.com/cmusphinx/cmudict) for\ndetails on the data. Chances are, if you're here, you already know what's in the\nfiles.\n\nEach file can be accessed through three functions, one which returns the raw\n(string) contents, one which returns a binary stream of the file, and one which\ndoes minimal processing of the file into an appropriate structure:\n\n```python\n>>> import cmudict\n\n>>> cmudict.dict() # Compatible with NLTK\n>>> cmudict.dict_string()\n>>> cmudict.dict_stream()\n\n>>> cmudict.phones()\n>>> cmudict.phones_string()\n>>> cmudict.phones_stream()\n\n>>> cmudict.symbols()\n>>> cmudict.symbols_string()\n>>> cmudict.symbols_stream()\n\n>>> cmudict.vp()\n>>> cmudict.vp_string()\n>>> cmudict.vp_stream()\n```\n\nThree additional functions are included to maintain compatibility with NLTK:\ncmudict.entries(), cmudict.raw(), and cmudict.words(). See the\n[nltk.corpus.reader.cmudict](http://www.nltk.org/_modules/nltk/corpus/reader/cmudict.html)\ndocumentation for details:\n\n```python\n>>> cmudict.entries() # Compatible with NLTK\n>>> cmudict.raw() # Compatible with NLTK\n>>> cmudict.words() # Compatible with NTLK\n```\n\nAnd finally, the license for the cmudict data set is available as well:\n\n```python\n>>> cmudict.license_string() # Returns the cmudict license as a string\n```\n\n## Credits\n\nBuilt on or modeled after the following open source projects:\n\n- [The CMU Pronouncing Dictionary](https://github.com/cmusphinx/cmudict)\n- [NLTK](https://github.com/nltk/nltk)\n",
    'author': 'David L. Day',
    'author_email': 'david@davidlday.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/prosegrinder/python-cmudict',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
